unit MAINDEV;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
  Button1  : TButton;      (* RUN button                           *)
  Button2  : TButton;      (* QUIT button                          *)
  Button3  : TButton;      (* ABOUT button                         *)
  Label1   : TLabel;
    ProgressBar1: TProgressBar;
    SaveDialog1: TSaveDialog;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    Separat: TRadioGroup;
    Special: TEdit;
    procedure Button2Click(Sender: TObject); (* Procedure to Exit.          *)
    procedure Button1Click(Sender: TObject); (* Procedure to run test.      *)
    procedure Button3Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SeparatClick(Sender: TObject); (* Procedure to display ABOUT. *)
private
  { Private declarations }
public
  { Public declarations }

  end;

const
  (* GPIB status bit definitions.                                        *)
  ERR     = $8000;           (* Error detected                           *)
  TIMO    = $4000;           (* Timeout                                  *)
  ENDgpib = $2000;           (* EOI or EOS detected                      *)
  SRQI    = $1000;           (* SRQ detected by CIC                      *)
  RQS     =  $800;           (* Device needs service                     *)
  SPOLL   =  $400;           (* Board has been serially polled           *)
  EVENT   =  $200;           (* An event has occurred                    *)
  CMPL    =  $100;           (* I/O completed                            *)
  LOK     =   $80;           (* Local lockout state                      *)
  REM     =   $40;           (* Remote state                             *)
  CIC     =   $20;           (* Controller-in-charge                     *)
  ATN     =   $10;           (* Attention asserted                       *)
  TACS    =    $8;           (* Talker active                            *)
  LACS    =    $4;           (* Listener active                          *)
  DTAS    =    $2;           (* Device trigger state                     *)
  DCAS    =    $1;           (* Device clear state                       *)

  (* Error messages returned in global variable IBERR:                   *)
  EDVR =  0;                 (* System error                             *)
  ECIC =  1;                 (* Function requires GPIB board to be CIC   *)
  ENOL =  2;                 (* Write function detected no Listeners     *)
  EADR =  3;                 (* Interface board not addressed correctly  *)
  EARG =  4;                 (* Invalid argument to function call        *)
  ESAC =  5;                 (* Function requires GPIB board to be SAC   *)
  EABO =  6;                 (* I/O operation aborted                    *)
  ENEB =  7;                 (* Non-existent interface board             *)
  EDMA =  8;                 (* Error performing DMA                     *)
  EOIP = 10;                 (* I/O operation started before previous    *)
			     (* operation completed                      *)
  ECAP = 11;                 (* No capability for intended operation     *)
  EFSO = 12;                 (* File system operation error              *)
  EBUS = 14;                 (* Command error during device call         *)
  ESTB = 15;                 (* Serial poll status byte lost             *)
  ESRQ = 16;                 (* SRQ remains asserted                     *)
  ETAB = 20;                 (* The return buffer is full                *)

  (* Timeout values and meanings:                                        *)
  TNONE   =  0;              (* Infinite timeout (disabled)              *)
  T10us   =  1;              (* Timeout of 10 microseconds (ideal)       *)
  T30us   =  2;              (* Timeout of 30 microseconds (ideal)       *)
  T100us  =  3;              (* Timeout of 100 microseconds (ideal)      *)
  T300us  =  4;              (* Timeout of 300 microseconds (ideal)      *)
  T1ms    =  5;              (* Timeout of 1 milliseconds (ideal)        *)
  T3ms    =  6;              (* Timeout of 3 milliseconds (ideal)        *)
  T10ms   =  7;              (* Timeout of 10 milliseconds (ideal)       *)
  T30ms   =  8;              (* Timeout of 30 milliseconds (ideal)       *)
  T100ms  =  9;              (* Timeout of 100 milliseconds (ideal)      *)
  T300ms  = 10;              (* Timeout of 300 milliseconds (ideal)      *)
  T1s     = 11;              (* Timeout of 1 second (ideal)              *)
  T3s     = 12;              (* Timeout of 3 seconds (ideal)             *)
  T10s    = 13;              (* Timeout of 10 seconds (ideal)            *)
  T30s    = 14;              (* Timeout of 30 seconds (ideal)            *)
  T100s   = 15;              (* Timeout of 100 seconds (ideal)           *)
  T300s   = 16;              (* Timeout of 300 seconds (ideal)           *)
  T1000s  = 17;              (* Timeout of 1000 seconds (ideal)          *)

type
  (* Type declarations for exported NI-488.2 Global Variables.           *)
  Tibsta =  function : integer; stdcall;
  Tiberr =  function : integer; stdcall;
  Tibcntl = function : Longint; stdcall;

  (* Type declarations for exported NI-488 functions.                    *)
  Tibclr = function (ud : integer) : integer; stdcall;

  Tibdev = function(ud:  integer;
                    pad: integer;
                    sad: integer;
                    tmo: integer;
                    eot: integer;
                    eos: integer) : integer; stdcall;

  Tibonl = function(ud: integer;
                    v:  integer) : integer; stdcall;

  Tibrd = function (ud:  integer;
                    var  rdbuf;
                    cnt: Longint) : integer; stdcall;

  Tibrsp = function (ud:      integer;
                     var spr: byte) : integer; stdcall;

  Tibtrg = function (ud : integer) : integer; stdcall;

  Tibwait = function (ud:   integer;
                      mask: integer) : integer; stdcall;

  Tibwrt = function (ud:  integer;
                     var  wrtbuf;
                     cnt: longint) : integer; stdcall;

  var
    Form1: TForm1;

(* Addresses for NI-488.2 GPIB global status variables                   *)
  AddrIbsta  : Tibsta;
  AddrIberr  : Tiberr;
  AddrIbcntl : Tibcntl;

(* Pointers to the NI-488.2 GPIB global status variables                 *)
  Pibsta  : ^integer;
  Piberr  : ^integer;
  Pibcntl : ^Longint;

(* Declaration for the Handle for the GPIB library                       *)
  Gpib32Lib: THandle;

(* Declarations for the NI-488 GPIB calls                                *)
  ibclr  : Tibclr;
  ibdev  : Tibdev;
  ibonl  : Tibonl;
  ibrd   : Tibrd;
  ibrsp  : Tibrsp;
  ibtrg  : Tibtrg;
  ibwait : Tibwait;
  ibwrt  : Tibwrt;

  dvm, dvm2    : integer;             (* Device number                         *)

implementation

{$R *.DFM}


(* =====================================================================
 *                      Procedure loadDLL
 * This procedure loads the GPIB-32.DLL library. If the LoadLibrary
 * call is successful, the next step is to get the addresses of the
 * global status variables and functions using GetProcAddress. If the
 * GetProcAddress calls were successful, the procedure returns to the
 * main routine. Otherwise, it will free the library and HALT.
 *
 * The HALT function will terminate this program.
 * ===================================================================== *)
procedure loadDLL;
var
  str : string;
  I : integer;

  begin
  (* Load the GPIB-32.DLL library using the LoadLibrary function.        *)
    Gpib32Lib := LoadLibrary('GPIB-32.dll');
    I:=GetLastError;

  (* Check to see if library loaded successfully. If the library could   *)
  (* not be loaded, display an error message and then HALT the program.  *)
    If Gpib32Lib = 0 Then
      Begin
        str := 'LoadLibrary FAILED!  '+intTostr(I);
        MessageDlg(str, mtError, [mbOK], 0);
        halt;
      End;

  (* Get the addresses of the GPIB Global Variables.                     *)
    @AddrIbsta  := GetProcAddress(Gpib32Lib, 'user_ibsta');
    @AddrIberr  := GetProcAddress(Gpib32Lib, 'user_iberr');
    @AddrIbcntl := GetProcAddress(Gpib32Lib, 'user_ibcnt');

  (* Get the addresses of the functions needed for this application.     *)
    @ibclr  := GetProcAddress(Gpib32Lib, 'ibclr');
    @ibdev  := GetProcAddress(Gpib32Lib, 'ibdev');
    @ibonl  := GetProcAddress(Gpib32Lib, 'ibonl');
    @ibrd   := GetProcAddress(Gpib32Lib, 'ibrd');
    @ibrsp  := GetProcAddress(Gpib32Lib, 'ibrsp');
    @ibtrg  := GetProcAddress(Gpib32Lib, 'ibtrg');
    @ibwait := GetProcAddress(Gpib32Lib, 'ibwait');
    @ibwrt  := GetProcAddress(Gpib32Lib, 'ibwrt');

  (* Verify that addresses were obtained. If unable to get any one of    *)
  (* the addresses, then free the library, display an error message and  *)
  (* HALT the program.                                                   *)
    if (@AddrIbsta  = NIL) Or
       (@AddrIberr  = NIL) Or
       (@AddrIbcntl = NIL) Or
       (@ibclr      = NIL) Or
       (@ibdev      = NIL) Or
       (@ibonl      = NIL) Or
       (@ibrd       = NIL) Or
       (@ibrsp      = NIL) Or
       (@ibtrg      = NIL) Or
       (@ibwait     = NIL) Or
       (@ibwrt      = NIL) Then
       Begin
         str := 'GetProcAddress FAILED!';
         MessageDlg(str, mtError, [mbOK], 0);
         (* Free the GPIB library.                                       *)
         FreeLibrary(Gpib32Lib);
         halt;
       End;

  (* Initialize GPIB global pointers to point to address location        *)
       Pibsta := @AddrIbsta;
       Piberr := @AddrIberr;
       Pibcntl := @AddrIbcntl;
  end;


(* =====================================================================
 *                      Procedure DVMERR
 * This procedure will notify you that the Fluke 45 returned an invalid
 * serial poll response byte. The error message will be printed along
 * with the serial poll response byte in a pop up message box.
 *
 * The NI-488 function ibonl is called to disable the hardware and
 * software.
 *
 * The HALT function will terminate this program.
 * ===================================================================== *)
procedure dvmerr(msg: string; spr: byte);
  var
    str      : string;       (* String used for displaying messages.     *)
    tempStr  : string;       (* String used for conversions.             *)

  begin
    tempStr := IntToHex(spr, 2);
    str := msg + '  Status Byte = $' + tempStr;
    MessageDlg(str, mtError, [mbOK], 0);

  (* Call the ibonl function to disable the hardware and software.    *)
    ibonl(dvm, 0);
  (* Free the GPIB library.                                           *)
    FreeLibrary(Gpib32Lib);

    halt;
  end;


(* =====================================================================
 *                       Procedure GPIBERR
 * This procedure will notify you that a NI-488 function failed by
 * printing an error message. The status variable IBSTA will be printed
 * in hexadecimal along with the mnemonic meaning of the bit position. The
 * status variable IBERR will be printed in decimal along with the
 * mnemonic meaning. The status variable IBCNTL will be printed in decimal.
 *
 * The NI-488 function ibonl is called to disable the hardware and
 * software.
 *
 * The HALT command stops execution of this program.
 * ===================================================================== *)
procedure gpiberr(msg: string);
  var
    str       : string;      (* String used for displaying messages.     *)
    ibstaStr  : string;      (* String for converting IBSTA.             *)
    iberrStr  : string;      (* String for converting IBERR.             *)
    ibcntlStr : string;      (* String for converting IBCNTL.            *)

  begin
    ibstaStr := IntToHex(Pibsta^, 4);
    iberrStr := IntToStr(Piberr^);
    str := msg + '  ibsta = $' + ibstaStr + '  iberr = ' + iberrStr;
    MessageDlg(str, mtError, [mbOK], 0);

    str := 'ibsta ===> ';
    if (Pibsta^ and ERR) <> 0 Then
      str := Concat(str, '  ERR');
    if (Pibsta^ and TIMO) <> 0 Then
      str := Concat(str, '  TMO');
    if (Pibsta^ and ENDgpib) <> 0 Then
      str := Concat(str, '  END');
    if (Pibsta^ and SRQI) <> 0 Then
      str := Concat(str, '  SRQI');
    if (Pibsta^ and RQS) <> 0 Then
      str := Concat(str, '  RQS');
    if (Pibsta^ and SPOLL) <> 0 Then
      str := Concat(str, '  SPOLL');
    if (Pibsta^ and EVENT) <> 0 Then
      str := Concat(str, '  EVENT');
    if (Pibsta^ and CMPL) <> 0 Then
      str := Concat(str, '  CMPL');
    if (Pibsta^ and LOK) <> 0 Then
      str := Concat(str, '  LOK');
    if (Pibsta^ and REM) <> 0 Then
      str := Concat(str, '  REM');
    if (Pibsta^ and CIC) <> 0 Then
      str := Concat(str, '  CIC');
    if (Pibsta^ and ATN) <> 0 Then
      str := Concat(str, '  ATN');
    if (Pibsta^ and TACS) <> 0 Then
      str := Concat(str, '  TACS');
    if (Pibsta^ and LACS) <> 0 Then
      str := Concat(str, '  LACS');
    if (Pibsta^ and DTAS) <> 0 Then
      str := Concat(str, '  DTAS');
    if (Pibsta^ and DCAS) <> 0 Then
      str := Concat(str, '  DCAS');

    MessageDlg(str, mtError, [mbOK], 0);

    str := 'iberr ===> ';
    if Piberr^ = EDVR Then
      str := Concat(str, '  EDVR');
    if Piberr^ = ECIC Then
      str := Concat(str, '  ECIC');
    if Piberr^ = ENOL Then
      str := Concat(str, '  ENOL');
    if Piberr^ = EADR Then
      str := Concat(str, '  EADR');
    if Piberr^ = EARG Then
      str := Concat(str, '  EARG');
    if Piberr^ = ESAC Then
      str := Concat(str, '  ESAC');
    if Piberr^ = EABO Then
      str := Concat(str, '  EABO');
    if Piberr^ = ENEB Then
      str := Concat(str, '  ENEB');
    if Piberr^ = EDMA Then
      str := Concat(str, '  EDMA');
    if Piberr^ = EOIP Then
      str := Concat(str, '  EOIP');
    if Piberr^ = ECAP Then
      str := Concat(str, '  ECAP');
    if Piberr^ = EFSO Then
      str := Concat(str, '  EFSO');
    if Piberr^ = EBUS Then
      str := Concat(str, '  EBUS');
    if Piberr^ = ESTB Then
      str := Concat(str, '  ESTB');
    if Piberr^ = ESRQ Then
      str := Concat(str, '  ESRQ');
    if Piberr^ = ETAB Then
      str := Concat(str, '  ETAB');

    MessageDlg(str, mtError, [mbOK], 0);

    ibcntlStr := IntToStr(Pibcntl^);
    str := 'ibcntl = ' + ibcntlStr;
    MessageDlg(str, mtError, [mbOK], 0);

  (* Call the ibonl function to disable the hardware and software.       *)
    ibonl(dvm, 0);
  (* Free the GPIB library.                                              *)
    FreeLibrary(Gpib32Lib);

    halt;
  end;


(* =====================================================================
 *                    Procedure TForm1.Button2Click
 * This procedure is activated when the QUIT button is clicked. This
 * procedure terminates the program.
 * ===================================================================== *)
procedure TForm1.Button2Click(Sender: TObject);
begin
  Close;
end;


(* =====================================================================
 *                    Procedure TForm1.Button3Click
 * This procedure is activated when the ABOUT button is clicked. This
 * procedure displays a pop up message box with information stating
 * that this program is a GPIB sample program.
 * ===================================================================== *)

procedure TForm1.Button3Click(Sender: TObject);
begin
  MessageDlg('Programme de transfert depuis '+chr(13)+
             'le Nicolet NIC 2081'+chr(13)+
             'sous Win32 par GPIB',mtInformation,[mbOK],0);
end;


procedure TForm1.Button1Click(Sender: TObject);
  var
    buf     : packed array[0..100] of char; (* R/W buffer.               *)
    i       : integer;
    S, Separateur : string;
    C : char;
    Resultats : array[1..4096] of string;
    F : textfile;
    VNormBit, HNormBit : boolean;
    VZero, HZero, Memory : integer;
    VNorm, HNorm : double;

begin
  if SaveDialog1.Execute then begin
    assignfile(F,SaveDialog1.FileName);
    rewrite(F);
    loadDLL;

    case Separat.ItemIndex of
      0 : Separateur:=' ';
      1 : Separateur:=',';
      2 : Separateur:=';';
      3 : Separateur:=chr(9);
      4 : Separateur:=Special.Text;
    end;

    dvm := ibdev(0, 14, 0, T1S, 1, 0);
    if (dvm < 0) THEN gpiberr('1-ibdev error');
    ibclr(dvm);
    Refresh;
    Screen.Cursor:=crHourGlass;
    ProgressBar1.Visible:=true;
    for I:=1 to 4096 do begin
      ProgressBar1.Position:=I;
      ibrd(dvm,buf,7);
      if (Pibsta^ AND ERR) <> 0 THEN gpiberr('2-ibrd error');
      Resultats[I]:=Buf[0]+Buf[1]+Buf[2]+Buf[3]+Buf[4];
    end;
    dvm2:=ibdev(0, 15, 0, T10S, 1, 0);
    ibclr(dvm2);
    buf:='N0';
    ibwrt(dvm2,buf,2);
    S:='';
    for I:=1 to 27 do begin
      ibrd(dvm,C,1);
      S:=S+C;
    end;
    Writeln(F,'V-norm_bit'+Separateur+S[1]);
    Writeln(F,'H-norm_bit'+Separateur+S[2]);
    Writeln(F,'Memory_Fraction'+Separateur+S[3]);
    Writeln(F,'V-zero'+Separateur+S[4]+S[5]+S[6]+S[7]+S[8]);
    Writeln(F,'H-zero'+Separateur+S[9]+S[10]+S[11]+S[12]+S[13]);
    Writeln(F,'V-norm'+Separateur+S[14]+S[15]+S[16]+S[17]+S[18]+S[19]+S[20]);
    Writeln(F,'H-norm'+Separateur+S[21]+S[22]+S[23]+S[24]+S[25]+S[26]+S[27]);
    if not CheckBox1.Checked then begin
      VNormBit:=S[1]='1';
      HNormBit:=S[2]='1';
      Memory:=strToInt(S[3]);
      VZero:=strToInt(S[4]+S[5]+S[6]+S[7]+S[8]);
      HZero:=strToInt(S[9]+S[10]+S[11]+S[12]+S[13]);
      if S[18]=' ' then S[18]:='+';
      if S[25]=' ' then S[25]:='+';
      VNorm:=strToFloat(S[14]+S[15]+S[16]+S[17]+S[18]+S[19]+S[20]);
      HNorm:=strToFloat(S[21]+S[22]+S[23]+S[24]+S[25]+S[26]+S[27]);
    end;
    if CheckBox2.Checked then begin
      for I:=1 to 2048 do begin
        if CheckBox1.Checked then begin
          S:=Resultats[2*I-1]+Separateur+Resultats[2*I];
          writeln(F,S);
        end
        else begin
          S:=FloatToStr(((I-1)-HZero)*Hnorm)+Separateur+
             FloatToStr((strToInt(Resultats[2*I-1])-VZero)*VNorm)+Separateur+
             FloatToStr((strToInt(Resultats[2*I])-VZero)*VNorm);
          writeln(F,S);
        end;
      end;
    end
    else begin
      for I:=1 to 4096 do begin
        if CheckBox1.Checked then begin
          S:=Resultats[I];
          writeln(F,S);
        end
        else begin
          S:=FloatToStr((I-1-HZero)*Hnorm)+Separateur+
             FloatToStr((strToInt(Resultats[I])-VZero)*VNorm);
          writeln(F,S);
        end;
      end;
    end;
    closeFile(F);
    ProgressBar1.Visible:=false;
    ibclr(dvm);
    ibclr(dvm2);
    ibonl(dvm,0);
    ibonl(dvm2,0);
    Screen.Cursor:=crDefault;
  end;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Button3.Top:=Height-74;
  Button1.Top:=Height-74;
  Button2.Top:=Height-74;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Gpib32Lib <> 0 then begin
    FreeLibrary(Gpib32Lib);
  end;
end;

procedure TForm1.SeparatClick(Sender: TObject);
begin
  Special.Enabled:=Separat.ItemIndex=4;
end;

end.


